/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __houdini_SurfaceAccessibleHoudini_h__
#define __houdini_SurfaceAccessibleHoudini_h__

namespace fe
{

/**************************************************************************//**
    @brief Houdini Surface Binding

	@ingroup houdini
*//***************************************************************************/
class FE_DL_EXPORT SurfaceAccessibleHoudini : virtual public SurfaceAccessibleI
{
	public:
								SurfaceAccessibleHoudini(void):
									m_pGdp(NULL)							{}

virtual							~SurfaceAccessibleHoudini(void)				{}

								//* as SurfaceAccessibleI

virtual sp<SurfaceAccessorI>	accessor(SurfaceAccessibleI::Element a_element,
										String a_name)
								{
									if(!m_pGdp)
									{
										return sp<SurfaceAccessorI>();
									}
									sp<SurfaceAccessorHoudini> spAccessor(
											new SurfaceAccessorHoudini);
									spAccessor->setGdp(m_pGdp);
									spAccessor->bind(a_element,a_name);
									return spAccessor;
								}

virtual sp<SurfaceAccessorI>	accessor(SurfaceAccessibleI::Element a_element,
										SurfaceAccessibleI::Attribute
										a_attribute)
								{
									if(!m_pGdp)
									{
										return sp<SurfaceAccessorI>();
									}
									sp<SurfaceAccessorHoudini> spAccessor(
											new SurfaceAccessorHoudini);
									spAccessor->setGdp(m_pGdp);
									spAccessor->bind(a_element,a_attribute);
									return spAccessor;
								}

virtual	sp<SurfaceI>			surface(void)
								{
									if(m_spSurfaceI.isNull())
									{
										m_spSurfaceI=registry()->create(
												"SurfaceI."
												"SurfaceTrianglesHoudini");
										if(m_pGdp && m_spSurfaceI.isValid())
										{
											sp<SurfaceTrianglesHoudini>
													spSurface=m_spSurfaceI;
											if(spSurface.isValid())
											{
												spSurface->setGdp(m_pGdp);
											}
										}
									}

									return m_spSurfaceI;
								}

virtual	void					append(sp<SurfaceAccessibleI>
										a_spSurfaceAccessibleI,
										SpatialTransform& a_rTransform);

virtual	void					instance(sp<SurfaceAccessibleI>
										a_spSurfaceAccessibleI,
										std::vector<SpatialTransform>&
										a_rTransformArray);

								//* Houdini specific
		void					setGdp(const GU_Detail* a_pGdp)
								{
									m_pGdp=a_pGdp;

									if(m_spSurfaceI.isValid())
									{
										sp<SurfaceTrianglesHoudini> spSurface=
												m_spSurfaceI;
										if(spSurface.isValid())
										{
											spSurface->setGdp(a_pGdp);
										}
									}
								}
const	GU_Detail*				gdp(void)					{ return m_pGdp; }

	private:
const	GU_Detail*		m_pGdp;
		sp<SurfaceI>	m_spSurfaceI;
};

} // namespace

#endif /* __houdini_SurfaceAccessibleHoudini_h__ */
