/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "houdini.pmh"

namespace fe
{

SurfaceTrianglesHoudini::SurfaceTrianglesHoudini(void)
{
	feLog("SurfaceTrianglesHoudini()\n");
}

SurfaceTrianglesHoudini::~SurfaceTrianglesHoudini(void)
{
}

Protectable* SurfaceTrianglesHoudini::clone(Protectable* pInstance)
{
	feLog("SurfaceTrianglesHoudini::clone\n");

	SurfaceTrianglesHoudini* pSurfaceTrianglesHoudini= pInstance?
			dynamic_cast<SurfaceTrianglesHoudini*>(pInstance):
			new SurfaceTrianglesHoudini();

	SurfaceTriangles::clone(pSurfaceTrianglesHoudini);

	// TODO copy attributes

	return pSurfaceTrianglesHoudini;
}

void SurfaceTrianglesHoudini::cache(void)
{
	feLog("SurfaceTrianglesHoudini::cache\n");

	//* NOTE not doing any Record serialization

	//* convert Houdini to vertex and normal arrays

	const U32 count=m_pGdp->primitives().entries();
	m_vertices=count*3;
	m_pVertexArray=(SpatialVector*)reallocate(m_pVertexArray,
			(m_vertices+1)*sizeof(SpatialVector));
	m_pNormalArray=(SpatialVector*)reallocate(m_pNormalArray,
			(m_vertices+1)*sizeof(SpatialVector));

	GEO_AttributeHandle handlePoint=m_pGdp->getAttribute(GEO_POINT_DICT,"N");

	U32 total=0;
	for(U32 index=0;index<count;index++)
	{
		const GEO_Primitive* pPrimitive=m_pGdp->primitives()(index);

		const U32 subCount=pPrimitive->getVertexCount();
		if(subCount!=3)
		{
			feLog("SurfaceTrianglesHoudini::cache"
					" primitive %d/%d subCount=%d\n",index,count,subCount);
		}
		FEASSERT(subCount==3);
		for(U32 subIndex=0;subIndex<subCount;subIndex++)
		{
			UT_Vector4 utVector4=
					pPrimitive->getVertex(subIndex).getPt()->getPos();
			m_pVertexArray[total]=SpatialVector(utVector4.data());

			const U32 pointIndex=
					pPrimitive->getVertex(subIndex).getBasePt()->getNum();

			const GEO_Point* pPoint=m_pGdp->points()(pointIndex);
			handlePoint.setElement(pPoint);
			utVector4=handlePoint.getV4(0);
			m_pNormalArray[total]=SpatialVector(utVector4.data());

			// HACK
//			m_pNormalArray[total]=
//					unit(m_pVertexArray[total]-SpatialVector(0.0,2.0,0.0));

			total++;
		}
	}
	FEASSERT(total==m_vertices);

	calcBoundingSphere();
}

void SurfaceTrianglesHoudini::calcBoundingSphere(void)
{
	if(!m_vertices)
	{
		return;
	}

	m_center=m_pVertexArray[0];
	for(U32 m=1;m<m_vertices;m++)
	{
		m_center+=m_pVertexArray[m];
	}
	m_center*=1.0/m_vertices;

	Real max2=0.0;
	for(U32 m=1;m<m_vertices;m++)
	{
		SpatialVector radial=m_pVertexArray[m]-m_center;
		Real dist2=magnitudeSquared(radial);
		if(max2<dist2)
		{
			max2=dist2;
		}
	}
	m_radius=sqrt(max2);

	feLog("SurfaceTrianglesHoudini::calcBoundingSphere center %s radius %.6G\n",
			c_print(m_center),m_radius);

}

void SurfaceTrianglesHoudini::cacheResolve(void)
{
	feLog("SurfaceTrianglesHoudini::cacheResolve m_vertices=%d tree valid=%d\n",
			m_vertices,m_spSpatialTreeI.isValid());

	if(!m_vertices)
	{
		return;
	}
	if(!m_spSpatialTreeI.isValid())
	{
		m_spSpatialTreeI=create("SpatialTreeI.default");
	}
	if(!m_spSpatialTreeI.isValid())
	{
		return;
	}

	feLog("SurfaceTrianglesHoudini::cacheResolve building spatial tree\n");
	m_spSpatialTreeI->populate(m_pVertexArray,m_pNormalArray,
			m_vertices,center(),radius());

	feLog("SurfaceTrianglesHoudini::cacheResolve done\n");
}

} // namespace
