#include "operator/operator.pmh"

namespace fe
{

void OperatorSurfaceSummary::initialize(void)
{
	catalog< sp<Component> >("Input Surface");

	catalog<I32>("pointCount");
	catalog<String>("pointCount","IO")="output";
	catalog<String>("pointCount","label")="Points";

	catalog<I32>("primitiveCount");
	catalog<String>("primitiveCount","IO")="output";
	catalog<String>("primitiveCount","label")="Primitives";

	catalog<String>("probeAttribute")="P";
	catalog<String>("probeAttribute","label")="Probe Attribute";

	catalog<String>("probeClass")="Point";
	catalog<String>("probeClass","label")="Probe Class";

	catalog<String>("probeType")="Real";
	catalog<String>("probeType","label")="Probe Type";

	catalog<I32>("probeIndex")=0;
	catalog<I32>("probeIndex","high")=100;
	catalog<I32>("probeIndex","max")=INT_MAX;
	catalog<String>("probeIndex","label")="Probe Index";

	catalog<Real>("probeValue")=0.0;
	catalog<String>("probeValue","IO")="output";
	catalog<String>("probeValue","label")="Probe Value";
}

void OperatorSurfaceSummary::handle(Record& a_rSignal)
{
	feLog("OperatorSurfaceSummary::handle\n");

	I32& pointCount=catalog<I32>("pointCount");
	I32& primitiveCount=catalog<I32>("primitiveCount");

	const String probeClass=catalog<String>("probeClass");
	const String probeType=catalog<String>("probeType");
	const String probeAttribute=catalog<String>("probeAttribute");
	const I32 probeIndex=catalog<I32>("probeIndex");
	Real& probeValue=catalog<Real>("probeValue");

	sp<SurfaceAccessibleI> spSurfaceAccessibleI=surfaceOutput(a_rSignal);
	feLog("OperatorSurfaceSummary::handle spSurfaceAccessibleI valid=%d\n",
			spSurfaceAccessibleI.isValid());
	FEASSERT(spSurfaceAccessibleI.isValid());

	sp<SurfaceAccessorI> spAccessor=spSurfaceAccessibleI->accessor(
			SurfaceAccessibleI::e_point,SurfaceAccessibleI::e_generic);
	FEASSERT(spAccessor.isValid());

	pointCount=spAccessor->count();
	feLog("OperatorSurfaceSummary::handle pointCount=%d\n",pointCount);

	spAccessor=spSurfaceAccessibleI->accessor(
			SurfaceAccessibleI::e_primitive,SurfaceAccessibleI::e_generic);
	FEASSERT(spAccessor.isValid());

	primitiveCount=spAccessor->count();
	feLog("OperatorSurfaceSummary::handle primitiveCount=%d\n",primitiveCount);

	bool probePoint=(probeClass=="Point");
	bool probeReal=(probeType=="Real");
	I32 probeMax=probePoint? pointCount: primitiveCount;
	if(probeIndex<probeMax)
	{
		sp<SurfaceAccessibleI> spSurfaceInput=
				catalog< sp<Component> >("Input Surface");
		FEASSERT(spSurfaceInput.isValid());

		spAccessor=spSurfaceInput->accessor(probePoint?
				SurfaceAccessibleI::e_point: SurfaceAccessibleI::e_primitive,
				probeAttribute);
		if(spAccessor.isValid())
		{
			// TODO auto check type
			if(probeReal)
			{
				probeValue=spAccessor->real(probeIndex,0);
			}
			else
			{
				probeValue=spAccessor->integer(probeIndex,0);
			}
			feLog("OperatorSurfaceSummary::handle probeValue=%.6G\n",
					probeValue);
		}
	}
	else
	{
		probeValue= -1.0;
	}

	feLog("OperatorSurfaceSummary::handle done\n");
}

} // namespace
