/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "grass/grass.pmh"

using namespace fe;

void OperatorLengthCorrect::initialize(void)
{
	catalog<I32>("Iterations")=1;
	catalog< sp<Component> >("Curves");
	catalog< sp<Component> >("Reference Curves");
}

void OperatorLengthCorrect::handle(Record& a_rSignal)
{
	feLog("OperatorLengthCorrect::handle\n");

	I32 iterations=catalog<I32>("Iterations");

	//* access reference curves
	sp<SurfaceAccessibleI> spReference=
			catalog< sp<Component> >("Reference Curves");
	if(spReference.isNull())
	{
		feLog("OperatorLengthCorrect::handle"
				" reference surface not spSurfaceAccessibleI\n");
		return;
	}

	//* access output curves (copy of input curves)
	sp<SurfaceAccessibleI> spSurfaceAccessibleI=surfaceOutput(a_rSignal);
	if(spSurfaceAccessibleI.isNull())
	{
		feLog("OperatorLengthCorrect::handle"
				" output surface not spSurfaceAccessibleI\n");
		return;
	}

	sp<SurfaceAccessorI> spAccessorRef=spReference->accessor(
			SurfaceAccessibleI::e_primitive,SurfaceAccessibleI::e_vertices);
	FEASSERT(spAccessorRef.isValid());

	sp<SurfaceAccessorI> spAccessor=spSurfaceAccessibleI->accessor(
			SurfaceAccessibleI::e_primitive,SurfaceAccessibleI::e_vertices);
	FEASSERT(spAccessor.isValid());

	//* tweak curve points towards reference length
	const U32 count = spAccessor->count();
	for(U32 iteration=0;iteration<iterations;iteration++)
	{
		for(U32 index=0;index<count;index++)
		{
			const U32 subCount = spAccessor->subCount(index);
			const U32 subCountRef = spAccessorRef->subCount(index);
			if(subCount!=subCountRef)
			{
				feLog("OperatorLengthCorrect::handle"
						" primitive %d mismatched count %d vs %d ref\n",
						index,subCount,subCountRef);
				continue;
			}
			//* NOTE skip root point
			for(U32 subIndex=1;subIndex<subCount;subIndex++)
			{
				SpatialVector pointA=
						spAccessor->spatialVector(index,subIndex-1);
				SpatialVector pointB=
						spAccessor->spatialVector(index,subIndex);

				SpatialVector pointRefA=
						spAccessorRef->spatialVector(index,subIndex-1);
				SpatialVector pointRefB=
						spAccessorRef->spatialVector(index,subIndex);

				SpatialVector delta=pointB-pointA;
				Real length=magnitude(delta);
				Real lengthRef=magnitude(pointRefB-pointRefA);
				SpatialVector adjust=(lengthRef/length-1.0)*delta;

				if(subIndex>1)
				{
					pointA-=0.3*adjust;
					pointB+=0.7*adjust;

					spAccessor->set(index,subIndex-1,pointA);
					spAccessor->set(index,subIndex,pointB);
				}
				else
				{
					pointB+=adjust;
					spAccessor->set(index,subIndex,pointB);
				}
			}
		}
	}

	feLog("OperatorLengthCorrect::handle done\n");
}
